{$B-}
unit TestRE;

{
 simple test of regular expression

 Andrey Sorokin, Saint-Petersburg, Russia
 anso@mail.ru, anso@usa.net
 http://anso.virtualave.net/

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls,
  RegExpr, ExtCtrls;

type
  TfmTestRE = class(TForm)
    btnClose: TBitBtn;
    btnEnglishHelp: TBitBtn;
    btnRussianHelp: TBitBtn;
    grpRegExpr: TGroupBox;
    lblRegExpr: TLabel;
    edRegExpr: TEdit;
    btnTemplatePhonePiter: TSpeedButton;
    btnTemplatePhone: TSpeedButton;
    btnTemplatePassport: TSpeedButton;
    btnTemplateMail: TSpeedButton;
    btnTemplateInteger: TSpeedButton;
    btnTemplateRealNumber: TSpeedButton;
    btnTemplateURL: TSpeedButton;
    btnViewPCode: TBitBtn;
    lblInputString: TLabel;
    edInputString: TEdit;
    btnFindRegExprInFile: TSpeedButton;
    OpenDialog1: TOpenDialog;
    lblOffset: TLabel;
    edOffset: TEdit;
    tabsExamples: TPageControl;
    tabSubstitution: TTabSheet;
    tabReplace: TTabSheet;
    lblSubstitutionTemplate: TLabel;
    memSubstitutionTemplate: TMemo;
    memSubstitutionResult: TMemo;
    lblSubstitutionResult: TLabel;
    pnlSubstitutionComment: TPanel;
    lblSubstitutionComment: TLabel;
    chkModifierI: TCheckBox;
    btnTestString: TBitBtn;
    btnExecNext: TBitBtn;
    lblTestResult: TLabel;
    pnlReplaceComment: TPanel;
    lblReplaceComment: TLabel;
    lblReplaceResult: TLabel;
    memReplaceResult: TMemo;
    btnReplace: TBitBtn;
    lblReplaceString: TLabel;
    edReplaceString: TEdit;
    tabSplit: TTabSheet;
    pnlSplitComment: TPanel;
    lblSplitComment: TLabel;
    btnSplit: TBitBtn;
    lblSplitResult: TLabel;
    memSplitResult: TMemo;
    btnSaintPetersburg: TSpeedButton;
    chkModifierR: TCheckBox;
    btnTemplateRomanNumber: TSpeedButton;
    chkModifierS: TCheckBox;
    btnBackRef: TSpeedButton;
    procedure btnViewPCodeClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnTestStringClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnTemplatePhonePiterClick(Sender: TObject);
    procedure btnTemplatePhoneClick(Sender: TObject);
    procedure btnTemplatePassportClick(Sender: TObject);
    procedure btnTemplateMailClick(Sender: TObject);
    procedure btnTemplateIntegerClick(Sender: TObject);
    procedure btnTemplateRealNumberClick(Sender: TObject);
    procedure btnTemplateURLClick(Sender: TObject);
    procedure btnRussianHelpClick(Sender: TObject);
    procedure btnEnglishHelpClick(Sender: TObject);
    procedure btnFindRegExprInFileClick(Sender: TObject);
    procedure btnExecNextClick(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure btnSplitClick(Sender: TObject);
    procedure chkModifierIClick(Sender: TObject);
    procedure btnSaintPetersburgClick(Sender: TObject);
    procedure btnTemplateRomanNumberClick(Sender: TObject);
    procedure chkModifierSClick(Sender: TObject);
    procedure chkModifierRClick(Sender: TObject);
    procedure btnBackRefClick(Sender: TObject);
   private
    r : TRegExpr;
    procedure Compile;
    procedure ExecIt (AFindNext : boolean);
  end;

var
  fmTestRE: TfmTestRE;

implementation
{$R *.DFM}

uses PCode, FileViewer;

procedure TfmTestRE.FormCreate(Sender: TObject);
 begin
  r := TRegExpr.Create;

  // show effective values of modifiers
  chkModifierI.Checked := r.ModifierI;
  chkModifierR.Checked := r.ModifierR;
  chkModifierS.Checked := r.ModifierS;
 end;

procedure TfmTestRE.FormDestroy(Sender: TObject);
 begin
  if Assigned (r)
   then r.Free;
 end;

procedure TfmTestRE.btnViewPCodeClick(Sender: TObject);
 begin
  Compile;
  with TfmPseudoCodeViewer.Create (Application) do begin
    edSource.Text := r.Expression;
    Memo1.Lines.Text := r.Dump;
    ShowModal;
   end;
 end;

procedure TfmTestRE.btnCloseClick(Sender: TObject);
 begin
  Close;
 end;

procedure TfmTestRE.Compile;
 begin
  try
    // r.e. precompilation (then you assign Expression property,
    // TRegExpr automatically compiles the r.e.).
    // Note:
    //   if there are errors in r.e. TRegExpr will raise
    //   exception.

    r.Expression := edRegExpr.Text;

    except on E:Exception do begin // exception during r.e. compilation or execution
      if E is ERegExpr then
       if (E as ERegExpr).CompilerErrorPos > 0 then begin
         // compilation exception - show place of error
         edRegExpr.SetFocus;
         edRegExpr.SelStart := (E as ERegExpr).CompilerErrorPos - 1;
         edRegExpr.SelLength := 1;
        end;
      raise Exception.Create (E.Message); // continue exception processing
     end;
   end;
 end;

procedure TfmTestRE.ExecIt (AFindNext : boolean);
 var
  i : integer;
  res : boolean;
  off : integer;
 begin
  try
    // Assign r.e. to Expression property - it'll be
    // automatically compiled. If any errors occure,
    // ERegExpr will be raised
    Compile;

    // r.e. execution (finding r.e. occurences in input string)
    //
    // There are some examples of r.e. execution technics:
    // 1) Simple case (one r.e., many input strings)
    //      r.Exec (AInputString);
    //  Note
    //   if you don't need r.e. precompilation and only want
    //   to check one input string with one r.e. then you may
    //   use global routine:
    //      ExecRegExpr (ARegularExpression, AInputString);
    // 2) Search next match
    //      if r.Exec (AInputString) then // search first occurence
    //       REPEAT
    //        // loop body
    //       UNTIL not r.ExecNext; // search next occurences
    // 3) Search from any position
    //      r.InputString := AInputString;
    //      r.ExecPos (APositionOffset);

    if AFindNext
     then res := r.ExecNext // search next occurence. raise
                            // exception if no Exec call preceded
     else begin
       off := StrToInt (edOffset.Text); // raise exception if bad Offset
       if off = 1
        then res := r.Exec (edInputString.Text) // search from first position
        else begin // search from off position
          r.InputString := edInputString.Text;
          res := r.ExecPos (off);
         end;
      end;
    if res then begin // r.e. found
       // Show r.e. positions: 0 - whole r.e.,
       // 1 .. SubExprMatchCount - subexpressions.
       lblTestResult.Caption := IntToStr (r.MatchPos [0])
        + '-' + IntToStr (r.MatchPos [0] + r.MatchLen [0] - 1);
       for i := 1 to r.SubExprMatchCount do
        if r.MatchPos [i] > 0
         then lblTestResult.Caption := lblTestResult.Caption + ', '
           + '\' + IntToStr (i) + ':' + IntToStr (r.MatchPos [i])
           + '-' + IntToStr (r.MatchPos [i] + r.MatchLen [i] - 1);
       // Mark r.e. in input string
       edInputString.SetFocus;
       edInputString.SelStart := r.MatchPos [0] - 1;
       edInputString.SelLength := r.MatchLen [0];
       lblTestResult.Caption := 'Reg.expr found at '
        + lblTestResult.Caption;
       lblTestResult.Font.Color := clGreen;
       // Perform substitution - example of fill in template
       memSubstitutionResult.Text :=
        r.Substitute (PChar (memSubstitutionTemplate.Text));
      end
     else begin // r.e. not found
       lblTestResult.Caption := 'Regexpr. not found in string.';
       lblTestResult.Font.Color := clPurple;
       memSubstitutionResult.Text := 'Substitution is not performed';
      end;
    except on E:Exception do begin // exception during r.e. compilation or execution
      lblTestResult.Caption := 'Error: "' + E.Message + '"';
      lblTestResult.Font.Color := clRed;
      memSubstitutionResult.Text := 'Substitution is not performed';
     end;
   end;
 end;

procedure TfmTestRE.btnTestStringClick(Sender: TObject);
 begin
  ExecIt (false);
 end;

procedure TfmTestRE.btnExecNextClick(Sender: TObject);
 begin
  ExecIt (true);
 end;

procedure TfmTestRE.btnTemplatePhonePiterClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '\d{3}-?\d\d-?\d\d';
  edInputString.Text :=
   'The phone of AlkorSoft (project PayCash) is 329-4469 (St-Petersburg)';
  memSubstitutionTemplate.Text :=
   'Great payment system! E-cash is real ! Phone $&, PayCash.';
 end;

procedure TfmTestRE.btnTemplatePhoneClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '(\+\d *)?(\(\d+\) *)?\d+(-\d*)*';
  edInputString.Text :=
   'Long distance phone of AlkorSoft (PayCash) is +7(812) 329-44-69';
  memSubstitutionTemplate.Text :=
   'If you call outside of Saint-Petersburg (zone code $1, city code $2), use $& for calling.';
 end;

procedure TfmTestRE.btnTemplatePassportClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '([IVXCL]{3}-[-][-] *[N]? *)?\d{6}';
  edInputString.Text :=
   'The example of russian passport number: IVX- 123456';
  memSubstitutionTemplate.Text :=
   'Number: $&';
 end;

procedure TfmTestRE.btnTemplateMailClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '[_a-zA-Z\d\-\.]+@[_a-zA-Z\d\-]+(\.[_a-zA-Z\d\-]+)+';
  edInputString.Text :=
   'My e-mails is anso@mail.ru and anso@usa.net';
  memSubstitutionTemplate.Text :=
   'Please, all suggestions and bugreports mail'
   + 'to $& (Andrey Sorokin).';
 end;


procedure TfmTestRE.btnTemplateIntegerClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '^(?#Remove ''^'' if something may precede number)[+\-]?\d+$(?#And ''$'' if something may follow)';
  edInputString.Text :=
   'Here is the number 123. Something after number.';
  memSubstitutionTemplate.Text :=
   'The r.e. above expects only digits in the string,'
   + ' because simbols of start and end of string'
   + ' (''^'' and ''$'').'#$d#$a
   + 'So, you must remove all non digits symbols in input string'
   + ' or symbols ''^'' and ''$'' in r.e.';
 end;

procedure TfmTestRE.btnTemplateRealNumberClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '^[+\-]?\d+(\.\d+)?([eE][+\-]?\d+)?$';
  edInputString.Text :=
   'Remove this123.0e6and this';
  memSubstitutionTemplate.Text :=
   'The r.e. above checks the start and end of string'
   + ' (symbols ''^'' and ''$'' in r.e.).'#$d#$a
   + 'So, you must remove this symbols from r.e. or'
   + ' all symbols before and after real number in'
   + ' input string.';
 end;

procedure TfmTestRE.btnTemplateURLClick(Sender: TObject);
 begin
  edRegExpr.Text :=
   '(?i)(FTP|HTTP)://(' // protocol
   + '[_a-z\d\-]+(\.[_a-z\d\-]+)+' // TCP addr
   + ')((/[ _a-z\d\-\\\.]+)+)*'; // unix path
  edInputString.Text :=
   'Welcome to my homepage http://anso.da.ru. E-cash http://www.paycash.ru or http://195.5.138.172/default.htm!';
  memSubstitutionTemplate.Text :=
   'The protocol is $1'#$d#$a
   + 'the address is $2'#$d#$a
   + 'the whole URL of our site is $&.';
 end;

procedure TfmTestRE.btnSaintPetersburgClick(Sender: TObject);
 begin
  edRegExpr.Text := '((?i)Saint-)?Petersburg(?#move modifier outside subexpression and it''ll match first occurence in example)';
  edInputString.Text :=
   'Welcome to petersburg! Saint-Petersburg is a wondeful city - look at photos on my homepage http://anso.da.ru';
  memSubstitutionTemplate.Text := 'The name of city is $&.';
 end;

procedure TfmTestRE.btnTemplateRomanNumberClick(Sender: TObject);
 begin
  edRegExpr.Text := '^(?i)M*(D?C{0,3}|C[DM])(L?X{0,3}|X[LC])(V?I{0,3}|I[VX])$';
  edInputString.Text :=
   'MCMXCIX';
  memSubstitutionTemplate.Text := '1999 is $& in roman numbers.';
 end;

procedure TfmTestRE.btnBackRefClick(Sender: TObject);
 begin
  edRegExpr.Text := '(?i)<font size=([''"]?)([+-]?\d+)\1>';
  edInputString.Text :=
   '<font size="+1"> big </font> <font size=1> very small</font>';
  memSubstitutionTemplate.Text := 'font size = $2 (extracted from tag: "$&").';
 end;

procedure TfmTestRE.btnRussianHelpClick(Sender: TObject);
 begin
  Application.HelpFile := 'RegExpR.hlp';
  Application.HelpCommand (HELP_FINDER, 0);
 end;

procedure TfmTestRE.btnEnglishHelpClick(Sender: TObject);
 begin
  Application.HelpFile := 'RegExpE.hlp';
  Application.HelpCommand (HELP_FINDER, 0);
 end;

procedure TfmTestRE.btnFindRegExprInFileClick(Sender: TObject);
 var
  FileStr : TStrings;
 begin
  if OpenDialog1.Execute then begin
    FileStr := TStringList.Create;
    try
      FileStr.LoadFromFile (OpenDialog1.FileName);
      r.Expression := edRegExpr.Text;
        with TfmFileViewer.Create (Application) do begin
          Caption := Caption + ' ' + OpenDialog1.FileName;
          RichEdit1.Lines.AddStrings (FileStr);
          if r.Exec (FileStr.Text) then //###0.90
            REPEAT
             RichEdit1.SelStart := r.MatchPos [0] - 1;
             RichEdit1.SelLength := r.MatchLen [0];
             RichEdit1.SelAttributes.Color := clRed;
            UNTIL not r.ExecNext;
          RichEdit1.SelLength := 0;
          Show;
         end;
      finally FileStr.Free;
     end;
   end;
 end;


procedure TfmTestRE.btnReplaceClick(Sender: TObject);
 begin
  Compile;
  memReplaceResult.Text := r.Replace (edInputString.Text,
   edReplaceString.Text);
 end;

procedure TfmTestRE.btnSplitClick(Sender: TObject);
 begin
  Compile;
  memSplitResult.Lines.Clear;
  r.Split (edInputString.Text, memSplitResult.Lines);
 end;

procedure TfmTestRE.chkModifierIClick(Sender: TObject);
 begin
  r.ModifierI := chkModifierI.Checked;
  // You may use also
  //   r.ModifierStr := 'i';
  // or
  //   r.ModifierStr := '-i';
 end;

procedure TfmTestRE.chkModifierRClick(Sender: TObject);
 begin
  r.ModifierR := chkModifierR.Checked;
 end;

procedure TfmTestRE.chkModifierSClick(Sender: TObject);
 begin
  r.ModifierS := chkModifierS.Checked;
 end;

end.


